@@ -44,13 +44,53 @@ fn main_impl(matches: ArgMatches<'_>) -> Result<(), Box<dyn std::error::Error>>
 
     let chr_filter = Regex::new(matches.value_of("filter").unwrap_or(".*"))?;
 
-    if matches.values_of("dict-auto").is_some() {
-        d4_builder.set_dictionary(Dictionary::from_sample_bam(
-            input_path,
-            |chr, _size| chr_filter.is_match(chr),
-            matches.value_of("ref"),
-            min_mq,
-        )?);
+    let mut enable_compression = false;
+
+    if (!matches.is_present("dict_range") && !matches.is_present("dict-file"))
+        || matches.is_present("dict-auto")
+    {
+        match input_type {
+            InputType::Alignment => {
+                d4_builder.set_dictionary(Dictionary::from_sample_bam(
+                    input_path,
+                    |chr, _size| chr_filter.is_match(chr),
+                    matches.value_of("ref"),
+                    min_mq,
+                )?);
+            }
+            InputType::BiwWig => {
+                let fp = std::fs::metadata(input_path)?;
+                let bw_file = d4_bigwig::BigWigFile::open(input_path)?;
+
+                let genome_size: u64 = bw_file.chroms().into_iter().map(|(_, sz)| sz as u64).sum();
+
+                let file_size = fp.len();
+
+                if file_size < genome_size / 8 {
+                    d4_builder.set_dictionary(Dictionary::new_simple_range_dict(0, 1)?);
+                    enable_compression = true;
+                }
+            }
+            InputType::BedGraph => {
+                let genomes = parse_genome_file(
+                    matches
+                        .value_of("genome")
+                        .expect("Genome file is required for text file format"),
+                )?;
+                let genome_size: u64 = genomes.into_iter().map(|chr| chr.size as u64).sum();
+
+                let fp = std::fs::metadata(input_path)?;
+                let file_size = fp.len();
+
+                if file_size < genome_size {
+                    d4_builder.set_dictionary(Dictionary::new_simple_range_dict(0, 1)?);
+                    enable_compression = true;
+                }
+            }
+            _ => {
+                panic!("Unsupported input type")
+            }
+        }
     }
 
     d4_builder.set_filter(move |chr, _size| chr_filter.is_match(chr));
@@ -62,7 +102,7 @@ fn main_impl(matches: ArgMatches<'_>) -> Result<(), Box<dyn std::error::Error>>
 
     let reference = matches.value_of("ref");
 
-    let enable_compression = matches.is_present("deflate") || matches.is_present("sparse");
+    enable_compression |= matches.is_present("deflate") || matches.is_present("sparse");
     let compression_level: u32 = matches.value_of("deflate-level").unwrap_or("5").parse()?;
 
     match input_type {
